#!/usr/bin/env python
"""
_StreamersByRun_

T0Mon: this class is used in the run page [streamer file table]
"""

from T0.DAS.Database.Oracle.LongResultQuery import LongResultQuery 
from WMCore.Database.DBFormatter import DBFormatter

class StreamersByRun(LongResultQuery):
    sql =  """SELECT streamer.streamer_id AS streamer_id,
                        wmbs_file_details.lfn AS lfn,
                        wmbs_file_details.filesize AS filesize,
                        wmbs_file_details.events AS events,
                        wmbs_file_runlumi_map.run AS run_id,
                        wmbs_file_runlumi_map.lumi AS lumi_id,
                        stream.name AS stream
                 FROM streamer
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = streamer.streamer_id AND
                   wmbs_file_runlumi_map.run = %s
                 INNER JOIN stream ON
                   streamer.stream_id = stream.id
                 INNER JOIN wmbs_file_details ON
                   wmbs_file_details.id = streamer.streamer_id
               
                 ORDER BY streamer.streamer_id DESC
                 """
    def execute(self, run, uplimit, downlimit, conn = None, transaction = False):
        """
        create correct sql query and execute it

        """
        try:
            run = int(run)
        except ValueError:
            run = 0

        bindVars = { 'uplimit' :uplimit,
                     'downlimit' : downlimit } 

        if run > 0:
            sqlRunCheck = """:RUN"""
            bindVars.update( { 'RUN' : run } )
        else:
            sqlRunCheck = """(SELECT MAX(run_id) FROM run)"""

        self.sql=self.sql % sqlRunCheck

        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)

        return self.formatDict(result)


class StreamersByRun_count(DBFormatter):
   sql = """SELECT COUNT(*)
                 FROM streamer 
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = streamer.streamer_id AND
                   wmbs_file_runlumi_map.run = %s
                 """
   def execute(self, run, conn = None, transaction = False):
        """
        create correct sql query and execute it

        """
        try:
            run = int(run)
        except ValueError:
            run = 0

        bindVars = {}

        if run > 0:
            sqlRunCheck = """:RUN"""
            bindVars.update( { 'RUN' : run } )
        else:
            sqlRunCheck = """(SELECT MAX(run_id) FROM run)""" 

        self.sql=self.sql % sqlRunCheck

        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)

        return self.format(result)
