#!/usr/bin/env python
"""
_FirstConditionSafeRun_

returns the smallest run number which is not reco locked for
which there is no reco locked run with a larger run number

"""

__all__ = []
__revision__ = "$Id: FirstConditionSafeRun.py,v 1.2 2011/06/23 11:44:40 hufnagel Exp $"
__version__ = "$Revision: 1.2 $"

from WMCore.Database.DBFormatter import DBFormatter

class FirstConditionSafeRun(DBFormatter):

    sql = """WITH reco_locked AS (
               SELECT run_id AS run_id,
                      CASE
                        WHEN reco_started = 0 AND
                             ( close_time = 0 OR
                               close_time + reco_timeout - reco_lock_timeout > 0 ) THEN 0
                        ELSE 1
                      END AS locked
               FROM run
             )
             SELECT MIN(run_id) AS run_id
             FROM reco_locked
             WHERE locked = 0
             AND run_id > ( SELECT MAX(run_id)
                            FROM reco_locked
                            WHERE locked = 1 ) 
             """
    
    def format(self, result):
        return self.formatDict(result)
