#!/usr/bin/env python
"""
_IsRunExpressComplete_

Checks if a run has finished express processing
"""
__all__ = []
__revision__ = "$Id: IsRunExpressComplete.py,v 1.2 2011/06/20 15:11:44 hufnagel Exp $"
__version__ = "$Revision: 1.2 $"

from WMCore.Database.DBFormatter import DBFormatter


class IsRunExpressComplete(DBFormatter):
    
    def execute(self, run, conn = None, transaction = False):
        """
        _execute_

        """
        run = int(run)
        bindVars = { 'RUN_ID' : run }

        #
        # first check if run exists and/or if it's still active
        #
        sql = """SELECT close_time AS close_time
                 FROM run
                 WHERE run_id = :RUN_ID
                 """

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        if len(self.formatDict(result)) == 0:
            return False
        elif self.formatDict(result)[0]['close_time'] == 0:
            return False

        #
        # check if there are unused streamer files
        #
        sql = """SELECT COUNT(*) AS count
                 FROM streamer
                 INNER JOIN run_stream_style_assoc ON
                   run_stream_style_assoc.stream_id = streamer.stream_id AND
                   run_stream_style_assoc.run_id = streamer.run_id AND
                   run_stream_style_assoc.style_id = (SELECT id FROM processing_style WHERE name = 'Express')
                 WHERE checkForZeroState(streamer.used) = 0
                 AND streamer.run_id = :RUN_ID
                 """

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        if self.formatDict(result)[0]['count'] != 0:
            return False

        #
        # check if there are outstanding express jobs
        #
        sql = """SELECT COUNT(*) AS count
                 FROM wmbs_sub_files_acquired
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_sub_files_acquired.fileid AND
                   wmbs_file_runlumi_map.run = :RUN_ID
                 INNER JOIN wmbs_subscription ON
                   wmbs_subscription.id = wmbs_sub_files_acquired.subscription AND
                   wmbs_subscription.fileset = (SELECT id FROM wmbs_fileset WHERE name = 'Express')
                 """

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        if self.formatDict(result)[0]['count'] != 0:
            return False

        #
        # check if there are available files for expressmerge
        #
        sql = """SELECT COUNT(*) AS count
                 FROM wmbs_sub_files_available
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_sub_files_available.fileid AND
                   wmbs_file_runlumi_map.run = :RUN_ID
                 INNER JOIN wmbs_subscription ON
                   wmbs_subscription.id = wmbs_sub_files_available.subscription AND
                   wmbs_subscription.fileset = (SELECT id FROM wmbs_fileset WHERE name = 'ExpressMergeable')
                 """

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        if self.formatDict(result)[0]['count'] != 0:
            return False

        #
        # check if there are outstanding expressmerge jobs
        #
        sql = """SELECT COUNT(*) AS count
                 FROM wmbs_sub_files_acquired
                 INNER JOIN wmbs_file_runlumi_map ON
                   wmbs_file_runlumi_map.fileid = wmbs_sub_files_acquired.fileid AND
                   wmbs_file_runlumi_map.run = :RUN_ID
                 INNER JOIN wmbs_subscription ON
                   wmbs_subscription.id = wmbs_sub_files_acquired.subscription AND
                   wmbs_subscription.fileset = (SELECT id FROM wmbs_fileset WHERE name = 'ExpressMergeable')
                 """

        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        if self.formatDict(result)[0]['count'] != 0:
            return False

        return True
