#!/usr/bin/env python
"""
_StatsDQM_

T0Mon: This class is used in frontpage and in every run page for the statistical table
"""

from WMCore.Database.DBFormatter import DBFormatter

class StatsDQM(DBFormatter):

    def execute(self, run, conn = None, transaction = False):
        """
        create correct sql query and execute it

        """
        try:
            run = int(run)
        except ValueError:
            run = 0

        bindVars = {}

        if run > 0:
            sqlRunCheck = """INNER JOIN wmbs_file_runlumi_map ON
                               wmbs_file_runlumi_map.fileid = wmbs_job_assoc.fileid AND
                               wmbs_file_runlumi_map.run = :RUN
                             """
            bindVars.update( { 'RUN' : run } )
        else:
            sqlRunCheck = ""

        sql = """SELECT job_state_map.status AS status,
                        COUNT(*) AS count
                 FROM (       
                   SELECT wmbs_job.id AS id,
                          CASE
                            WHEN wmbs_job_state.name = 'cleanout' AND wmbs_job.outcome = 1 THEN 'complete'
                            WHEN wmbs_job_state.name = 'cleanout' AND wmbs_job.outcome = 0 THEN 'failed'
                            ELSE 'acquired'
                          END AS status
                   FROM wmbs_job
                   INNER JOIN wmbs_jobgroup ON
                     wmbs_jobgroup.id = wmbs_job.jobgroup AND
                     wmbs_jobgroup.subscription != (SELECT id FROM wmbs_subscription
                                                    WHERE fileset = (SELECT id FROM wmbs_fileset
                                                                     WHERE name = 'Mergeable')) AND
                     wmbs_jobgroup.subscription != (SELECT id FROM wmbs_subscription
                                                    WHERE fileset = (SELECT id FROM wmbs_fileset
                                                                     WHERE name = 'ExpressMergeable'))
                   INNER JOIN wmbs_job_state ON
                     wmbs_job_state.id = wmbs_job.state
                   INNER JOIN wmbs_job_assoc ON
                     wmbs_job_assoc.job = wmbs_job.id
                   INNER JOIN wmbs_file_dataset_path_assoc ON
                     wmbs_file_dataset_path_assoc.file_id = wmbs_job_assoc.fileid
                   INNER JOIN dataset_path ON
                     dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
                   INNER JOIN data_tier ON
                     data_tier.id = dataset_path.data_tier AND
                     data_tier.name = 'DQM'
                   %s
                   GROUP BY wmbs_job.id,
                            CASE
                              WHEN wmbs_job_state.name = 'cleanout' AND wmbs_job.outcome = 1 THEN 'complete'
                              WHEN wmbs_job_state.name = 'cleanout' AND wmbs_job.outcome = 0 THEN 'failed'
                              ELSE 'acquired'
                            END
                 ) job_state_map
                 GROUP BY job_state_map.status
                 """ % sqlRunCheck
   
        result = self.dbi.processData(sql, bindVars, conn = conn,
                                      transaction = transaction)

        resultDicts = self.formatDict(result)

        haveComplete = False
        haveFailed = False
        haveAcquired = False
        for resultDict in resultDicts:
            status = resultDict['status']
            if status == "complete":
                haveComplete = True
            elif status == "failed":
                haveFailed = True
            elif status == "acquired":
                haveAcquired = True

        if not haveComplete:
            resultDicts.append( { 'status' : 'complete',
                                  'count' : 0 } )
        if not haveFailed:
            resultDicts.append( { 'status' : 'failed',
                                  'count' : 0 } )
        if not haveAcquired:
            resultDicts.append( { 'status' : 'acquired',
                                  'count' : 0 } )

        return resultDicts
