from WMCore.HTTPFrontEnd.WorkQueue.Services.ServiceInterface import ServiceInterface
from WMCore.DAOFactory import *
from T0.DAS import OverviewCache
from threading import Thread
from T0.DAS import CachedDAOFactory

class MotT0Service(ServiceInterface):
    """
    _MotT0Service_
    
    Information provider for the MottT0 web UI  
    """
    defaultExpiresTime=300
    
    
    def register(self):
        # Use only CachedDAOFactory to load '*_count' DAOs
        daoFactory =  CachedDAOFactory.CachedDAOFactory(package = "T0.DAS.Database",
                                     logger = self.model,
                                     dbinterface = self.model.dbi, timeout=self.defaultExpiresTime)
        
        daoFactoryNotCached =  DAOFactory(package = "T0.DAS.Database",
                                     logger = self.model,
                                     dbinterface = self.model.dbi)
        #daoFactory=daoFactoryNotCached

        self.model.addDAO('GET', 'lastruns', 'Runs.LastRuns', daoFactory= daoFactoryNotCached)
        #heartbeats
        self.model.addDAO('GET', 'heartbeats', 'Components.HeartBeats', daoFactory= daoFactoryNotCached)
        self.model.addDAO('GET', 'heartbeatswithalarms', 'Components.HeartBeatsWithAlarms', daoFactory= daoFactoryNotCached)

        
        #statistics [general+ run]
        self.model.addDAO('GET', 'statsexpress', 'Stats.StatsExpress', args=['run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsexpressmerge', 'Stats.StatsExpressMerge', args=['run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsrepack', 'Stats.StatsRepack', args=['run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsrepackmerge', 'Stats.StatsRepackMerge',args=['run'], daoFactory= daoFactory) 
        self.model.addDAO('GET', 'statsreco', 'Stats.StatsReco',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsrecomerge', 'Stats.StatsRecoMerge',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsalca', 'Stats.StatsAlcaSkim', args=['run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'statsalcamerge', 'Stats.StatsAlcaMerge',args=['run'], daoFactory= daoFactory)
	self.model.addDAO('GET', 'statsdqm', 'Stats.StatsDQM',args=['run'], daoFactory= daoFactory)
        
        #run file details [main page top table]
        self.model.addDAO('GET', 'runfilesdetails', 'Runs.RunFilesDetails',args=['run'], daoFactory= daoFactory)

        #main page job details [general ]
        #  Express and Repack use  lastacquiredstreamerjobs
        self.model.addDAO('GET', 'lastacquiredstreamerjobs', 'Jobs.LastAcquiredStreamerJobs',args=['fileset','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'lastacquiredstreamerjobs_count', 'Jobs.LastAcquiredStreamerJobs_count',args=['fileset'], daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'expressmergelastjobs', 'Jobs.LastAcquiredExpressMerge',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'expressmergelastjobs_count', 'Jobs.LastAcquiredExpressMerge_count', daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'repackmergelastacquired', 'Jobs.LastAcquiredRepackMerge',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackmergelastacquired_count', 'Jobs.LastAcquiredRepackMerge_count', daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'recolastacquired', 'Jobs.LastAcquiredReco',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recolastacquired_count', 'Jobs.LastAcquiredReco_count', daoFactory= daoFactory)

        self.model.addDAO('GET', 'recomergelastacquired', 'Jobs.LastAcquiredRecoMerge',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recomergelastacquired_count', 'Jobs.LastAcquiredRecoMerge_count', daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'alcalastacquired', 'Jobs.LastAcquiredAlcaSkim',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcalastacquired_count', 'Jobs.LastAcquiredAlcaSkim_count', daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'alcamergelastacquired', 'Jobs.LastAcquiredAlcaMerge',args=['uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcamergelastacquired_count', 'Jobs.LastAcquiredAlcaMerge_count', daoFactory= daoFactory)
        
        #single pages details [general + run]
        self.model.addDAO('GET', 'expressjobsbystatus', 'Jobs.ExpressJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'expressjobsbystatus_count', 'Jobs.ExpressJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'expressmergejobsbystatus', 'Jobs.ExpressMergeJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'expressmergejobsbystatus_count', 'Jobs.ExpressMergeJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackjobsbystatus', 'Jobs.RepackJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackjobsbystatus_count', 'Jobs.RepackJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackmergejobsbystatus', 'Jobs.RepackMergeJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackmergejobsbystatus_count', 'Jobs.RepackMergeJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recojobsbystatus', 'Jobs.RecoJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recojobsbystatus_count', 'Jobs.RecoJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recomergejobsbystatus', 'Jobs.RecoMergeJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'recomergejobsbystatus_count', 'Jobs.RecoMergeJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcajobsbystatus', 'Jobs.AlcaSkimJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcajobsbystatus_count', 'Jobs.AlcaSkimJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcamergejobsbystatus', 'Jobs.AlcaMergeJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcamergejobsbystatus_count', 'Jobs.AlcaMergeJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
	
        self.model.addDAO('GET', 'dqmjobsbystatus', 'Jobs.DQMJobsByStatus',args=['status','run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'dqmjobsbystatus_count', 'Jobs.DQMJobsByStatus_count',args=['status','run'], daoFactory= daoFactory)
        
        #run inforomation
        
        #project configurations    ########## Express_config and Reco_config are DISABLED because they are already running in Tier0RecoService!!!
        self.model.addDAO('GET', 'express_config', 'Configs.ExpressStreamConfig',args=['run','stream'], daoFactory= daoFactory)
        #####self.model.addDAO('GET', 'repack_config', 'Configs.RepackDatasetConfig',args=['run','dataset'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'repack_config', 'Configs.RepackStreamConfig',args=['run','stream'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'reco_config', 'Configs.RecoDatasetConfig',args=['run','dataset'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'alca_config', 'Configs.AlcaDatasetConfig',args=['run','dataset'], daoFactory= daoFactory)
        #####self.model.addDAO('GET', 'general_config', 'Configs.GeneralConfig',args=['run'], daoFactory= daoFactory)
        
        #streamer & files
        self.model.addDAO('GET', 'runstreamers', 'Files.StreamersByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runstreamers_count', 'Files.StreamersByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtempexpressfiles', 'Files.TempExpressFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtempexpressfiles_count', 'Files.TempExpressFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runexpressfiles', 'Files.ExpressFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runexpressfiles_count', 'Files.ExpressFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtemprepackfiles', 'Files.TempRepackFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtemprepackfiles_count', 'Files.TempRepackFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runrepackfiles', 'Files.RepackFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runrepackfiles_count', 'Files.RepackFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtemprecofiles', 'Files.TempRecoFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtemprecofiles_count', 'Files.TempRecoFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runrecofiles', 'Files.RecoFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runrecofiles_count', 'Files.RecoFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtempalcafiles', 'Files.TempAlcaFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runtempalcafiles_count', 'Files.TempAlcaFilesByRun_count',args=['run'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runalcafiles', 'Files.AlcaFilesByRun',args=['run','uplimit','downlimit'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'runalcafiles_count', 'Files.AlcaFilesByRun_count',args=['run'], daoFactory= daoFactory)
        
        #historical tables!!!!
        self.model.addDAO('GET', 'allhistory', 'Components.AllHistory',args=['vtype','value'], daoFactory=daoFactory)
        
        #queries for front-page graphs
        
        #self.model.addDAO('GET', 'numtotalstreamers', 'Stats.NumTotalStreamersByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpressstreamersbyrun', 'Stats.NumExpressStreamersByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpresstotalstreamersbyrun', 'Stats.NumExpressTotalStreamersByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpresscompletedfiles', 'Stats.NumExpressCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpresstotalfiles', 'Stats.NumExpressTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpressmergedcompletedfiles', 'Stats.NumExpressMergedCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpressmergetotalfiles', 'Stats.NumExpressMergeTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        ## repack
        #self.model.addDAO('GET', 'numrepacktempbyrun', 'Stats.NumRepackTempByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrepackcompletedfiles', 'Stats.NumRepackCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrepackedtotalfiles', 'Stats.NumRepackTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrepackmergedcompletedfiles', 'Stats.NumRepackMergedCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrepackmergedexportedfiles', 'Stats.NumRepackMergedExportedFilesByRun',args=['run'], daoFactory= daoFactory)
        ##reco
        #self.model.addDAO('GET', 'numrecotempcompletedfiles', 'Stats.NumRecoTempCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrecotemptotalfiles', 'Stats.NumRecoTempTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrecocompletedfiles', 'Stats.NumRecoCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrecototalfiles', 'Stats.NumRecoTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrecomergedcompletedfiles', 'Stats.NumRecoMergedCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numrecomergedexportedfiles', 'Stats.NumRecoMergedExportedFilesByRun',args=['run'], daoFactory= daoFactory)
        ##alca
        #self.model.addDAO('GET', 'numalcatempcompletedfiles', 'Stats.NumAlcaTempCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numalcatemptotalfiles', 'Stats.NumAlcaTempTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numalcacompletedfiles', 'Stats.NumAlcaCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numalcatotalfiles', 'Stats.NumAlcaTotalFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numalcamergedcompletedfiles', 'Stats.NumAlcaMergedCompletedFilesByRun',args=['run'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numalcamergedexportedfiles', 'Stats.NumAlcaMergedExportedFilesByRun',args=['run'], daoFactory= daoFactory)
        
        
        #########new queries

        self.model.addDAO('GET', 'numstreamers', 'Stats.NumStreamers',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numexpressstreamers', 'Stats.NumExpressStreamers',args=['lowerrunid','higherrunid'], daoFactory= daoFactory) 
        self.model.addDAO('GET', 'numexpresstotalstreamers', 'Stats.NumExpressTotalStreamers',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numexprcompleted', 'Stats.NumExpressCompleted',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numexpresstotal', 'Stats.NumExpressTotal',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpressmerged', 'Stats.NumExpressMerged',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        #self.model.addDAO('GET', 'numexpressmergetotal', 'Stats.NumExpressMergeTotal',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        
          ## repack
        self.model.addDAO('GET', 'numrepacktemp', 'Stats.NumRepackTemp',args=['lowerrunid','higherrunid'], daoFactory= daoFactory) #compatibility reasons
        self.model.addDAO('GET', 'numrepackcompleted', 'Stats.NumRepackCompleted',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrepackedtotal', 'Stats.NumRepackTotal',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrepackmerged', 'Stats.NumRepackMerged',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrepackmergedexported', 'Stats.NumRepackMergedExported',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
         #reco
        self.model.addDAO('GET', 'numrecotemp', 'Stats.NumRecoTemp',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrecotemptotal', 'Stats.NumRecoTempTotal',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrecocompleted', 'Stats.NumRecoCompleted',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrecototal', 'Stats.NumRecoTotal',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrecomerged', 'Stats.NumRecoMerged',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numrecomergedexported', 'Stats.NumRecoMergedExported',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
         #alca
        self.model.addDAO('GET', 'numalcatemp', 'Stats.NumAlcaTemp',args=['lowerrunid','higherrunid'], daoFactory= daoFactory)
        self.model.addDAO('GET', 'numalcatemptotal', 'Stats.NumAlcaTempTotal',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numalcacompleted', 'Stats.NumAlcaCompleted',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numalcatotal', 'Stats.NumAlcaTotal',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numalcamerged', 'Stats.NumAlcaMerged',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        self.model.addDAO('GET', 'numalcamergedexported', 'Stats.NumAlcaMergedExported',args=['lowerrunid','higherrunid'],  daoFactory= daoFactory)
        
        
        ## Queries for  plots
        self.model.addDAO('GET', 'lastfilessummary', 'Files.LastFilesSummary', args=['stime','etime','tier'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'laststreamssummary', 'Streams.LastStreamsSummary', args=['stime','etime'],daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'expresslifetimes', 'Timing.ExpressJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'expressmergelifetimes', 'Timing.ExpressMergeJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'repacklifetimes', 'Timing.RepackJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'repackmergelifetimes', 'Timing.RepackMergeJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        
        self.model.addDAO('GET', 'recolifetimes', 'Timing.RecoJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'recomergelifetimes', 'Timing.RecoMergeJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcalifetimes', 'Timing.AlcaJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        self.model.addDAO('GET', 'alcamergelifetimes', 'Timing.AlcaMergeJobLifetime', args=['stime','etime','run'],daoFactory= daoFactory)
        #self.model.addDAO('GET', 'lastjobstimedatatier', 'Jobs.LastJobsTimeDataTier', args=['time'],daoFactory= daoFactory)
    #FIXME: it is possible to force here some parameters! 
    #def runLimitsValidation(self, input):
        #"""
        #validate parameters for: run + uplimit + downlimit
        #"""
        #if input.has_key("run") and input["run"]:
            #input["run"] = int(input["run"])
        #if input.has_key("uplimit") and input["uplimit"]:
            #input["uplimit"] = int(input["uplimit"])
        #if input.has_key("downlimit") and input["downlimit"]:
            #input["downlimit"] = int(input["downlimit"])
            
        #return input
